<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * IMAP, POP3 and NNTP
 * http://php.net/manual/en/book.imap.php
 *
 * @package HostCMS 6\Core\Mail
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Core_Mail_Imap extends Core_Servant_Properties
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array(
		'server', // сервер для соединения (IMAP, POP3)
		'port',
		'type', // imap, pop3
		'login',
		'password',
		'ssl', // SSL соединение
		'delete', // Удалять сообщения после получения
	);

	public function __construct()
	{
		parent::__construct();

		$this->delete = FALSE;
		$this->ssl = FALSE;
	}

	/**
	 * Протокол соединения
	 * @var string
	 */
	protected $_protocol = NULL;

	/**
	 * IMAP stream
	 * @var resource
	 */
	protected $_stream = NULL;

	/**
	 * Массив, содержащий письма
	 * @var array
	 */
	protected $_aMessages = array();

	public function getMessages()
	{
		return $this->_aMessages;
	}

	protected $_aErrors = array();

	public function getLastError()
	{
		return isset($this->_aErrors[0]) ? $this->_aErrors[0] : NULL;
	}

	public function getErrors()
	{
		return $this->_aErrors;
	}

	/**
	 * Заголовки письма
	 * @var str
	 */
	protected $_headers = '';

	public function execute()
	{
		// Если указан нестандартный порт, записываем его номер
		$aServer = explode(':', $this->server);

		$this->server = $aServer[0];
		is_null($this->port) && isset($aServer[1]) && $this->port = intval($aServer[1]);

		switch ($this->type)
		{
			case 'imap':
				is_null($this->port) && $this->port = 143;
				$this->_protocol = '/imap';
			break;
			case 'pop3':
				is_null($this->port) && $this->port = 110;
				$this->_protocol = '/pop3';
			break;
			default:
				throw new Core_Exception("Wrong type '%type', 'imap' and 'pop3' are possible.",
					array('%type' => $this->type)
				);
		}

		// Безопасное соединение TSL/SSL
		$this->_protocol .= $this->ssl
			? '/ssl'
			: '/novalidate-cert/notls';

		// Формируем имя ящика
		$mailbox = '{' . $this->server . ':' . $this->port . $this->_protocol . '}INBOX';

		//echo $this->login, '  *** ', $this->password, '  **** ', $mailbox, ' ';

		// Устанавливаем соединение с почтовым сервером
		$this->_stream = @imap_open($mailbox, $this->login, $this->password);

		// Соединение с почтовым сервером не установлено
		if (!$this->_stream)
		{
			$this->_aErrors = imap_errors();
			return $this;
		}

		// Количество писем в почтовом ящике
		$iCount = @imap_num_msg($this->_stream);

		if (!$iCount)
		{
			// imap_errors для пресечения вывода сообщений об ошибках, в том числе, если ящик пуст
			imap_errors();
			imap_close($this->_stream);
			return $this;
		}

		$i = 0;
		$this->_aMessages = array();

		while ($i < $iCount)
		{
			// Метаданные сообщения (структура)
			$structure = imap_fetchstructure($this->_stream, $i + 1);

			// Тип сообщения
			$this->_aMessages[$i]['type'] = $structure->subtype;

			// Кодировка, сообщение из нескольких частей
			if (isset($structure->parts))
			{
				// Кодировка для текстового тела сообщения
				foreach ($structure->parts[0]->parameters as $parameter)
				{
					if (strtolower($parameter->attribute) == 'charset')
					{
						$this->_aMessages[$i]['code'] = $parameter->value;
					}
				}

				// Метод кодировки сообщения
				if ($structure->parts[0]->encoding == 3)
				{
					$this->_aMessages[$i]['body']['encoding'] = 'base64';
				}
				elseif ($structure->parts[0]->encoding == 4)
				{
					$this->_aMessages[$i]['body']['encoding'] = 'quoted-printable';
				}
			}
			// Сообщение состоит из одной части
			else
			{
				$aStructureParts = array();

				foreach ($structure as $attribute => $attributeValue)
				{
					if (is_array($attributeValue))
					{
						// [parts][0][parameters][]
						foreach ($attributeValue as $parameter_num => $parameter_values)
						{
							if (!is_array($parameter_values))
							{
								// [parts][0][parameters][0][attribute]
								foreach ($parameter_values as $parameter_values_part_num => $parameter_values_part_values)
								{
									if (!is_array($parameter_values_part_values) && isset($parameter_values->attribute) && isset($parameter_values->value))
									{
										$aStructureParts[strtolower($parameter_values->attribute)] = $parameter_values->value;
									}
								}
							}
						}
					}
					else
					{
						$aStructureParts[strtolower($attribute)] = $attributeValue;
					}
				}

				// Кодировка всего сообщения
				$this->_aMessages[$i]['code'] = isset($aStructureParts['charset'])
					? $aStructureParts['charset'] : 'UTF-8';

				// Метод кодировки сообщения
				$this->_aMessages[$i]['encoding'] = isset($aStructureParts['parameters'])
					? $aStructureParts['encoding'] : 4;

				$this->_aMessages[$i]['structure'] = $this->_saveStructure($aStructureParts);
				$this->_aMessages[$i]['structure_array'] = $aStructureParts;

				// Подтип сообщения
				$this->_aMessages[$i]['subtype'] = isset($aStructureParts['subtype'])
					? $aStructureParts['subtype']
					: 'text';
			}

			// Тело сообщения, включая вложения
			$imap_body = imap_body($this->_stream, $i + 1);

			// Сообщение состоит из нескольких частей
			if (isset($structure->parts))
			{
				// Параметры письма
				$parameters = $structure->parameters;
				foreach ($parameters as $key => $value)
				{
					$bound = $value->value;
				}

				// Разбираем сообщение на части
				$this->_parseMessage($i, $imap_body, $bound);
			}
			// В сообщении одна только текстовая часть
			else
			{
				$this->_aMessages[$i]['body'] = isset($aStructureParts['encoding'])
					? $this->_bodyDecode(trim($imap_body), $aStructureParts['encoding'])
					: trim($imap_body);

				if (isset($this->_aMessages[$i]['code']))
				{
					$this->_aMessages[$i]['body'] = $this->_iconv($this->_aMessages[$i]['body'], $this->_aMessages[$i]['code']);
				}
			}

			$imap_fetchheader = imap_fetchheader($this->_stream, $i + 1);
			
			// Fix bug 'Fatal error: imap_headerinfo(): Address buffer overflow'
			// header can't be more 16K length
			if (strlen($imap_fetchheader) > 16384)
			{
				// skip broken message
				$i++;
			    continue;
			}

			// Заголовки письма
			$header_message = imap_headerinfo($this->_stream, $i + 1);

			!isset($this->_aMessages[$i]['code']) && $this->_aMessages[$i]['code'] = '';

			// Сохраняем заголовки
			$this->_headers = '';
			$this->_aMessages[$i]['headers'] = $this->_saveHeaders($header_message, $this->_aMessages[$i]['code']);

			// Заголовки письма
			$aImap_fetchheader = explode("\n", trim($imap_fetchheader));

			$fetchheader = array();
			foreach ($aImap_fetchheader as $key => $value)
			{
				$aValue = explode(":", $value);
				isset($aValue[1]) && $fetchheader[strtolower(trim(strval($aValue[0])))] = strtolower(trim(strval($aValue[1])));
			}

			$this->_aMessages[$i]['fetchheader'] = $fetchheader;
			$this->_aMessages[$i]['fetchheader_str'] = $this->_saveHeaders($fetchheader, $this->_aMessages[$i]['code']);

			// Дата письма
			$this->_aMessages[$i]['date'] = strftime("%d.%m.%Y %H:%M:%S", strtotime(
				isset($header_message->date) ? $header_message->date : $header_message->MailDate
			));

			if (isset($header_message->from))
			{
				// Узнаем отправителя
				$from = $header_message->from;
				foreach ($from as $key => $value)
				{
					// Почтовый ящик отправителя
					$this->_aMessages[$i]['from'] = $value->mailbox . "@" . $value->host;
				}
			}

			// Тему берем из email_imap_fetch_overview,
			// поскольку email_imap_header для этих целей по ряду причин может не сработать
			$subj = imap_fetch_overview($this->_stream, $i + 1);

			foreach ($subj as $key => $value)
			{
				if (isset($value->subject))
				{
					// Тема письма (если длинная, она кодируется частями)
					// Ищем в теме закодированные части, после чего раскодируем каждую отдельную часть темы, соединяя последовательно с незакодированными частями.
					$aSubject = array();

					$this->_aMessages[$i]['subject'] = '';

					// 000015012 eдаляем из темы [--Formal Message--], если появился
					if (mb_strpos($value->subject, '[--Formal Message--]') !== FALSE)
					{
						$value->subject = trim(str_replace('[--Formal Message--]', '', $value->subject));
					}

					preg_match_all("'(=\?([^=]*?)\?Q?\??(=?)([^\?]*?)\?=)'siu", $value->subject, $aSubject);

					// В теме есть закодированные части
					if (isset($aSubject[1]) && count($aSubject[1]))
					{
						$str_pos_start = mb_strpos($value->subject, $aSubject[1][0]);
						$this->_aMessages[$i]['subject'] = mb_substr($value->subject, 0, $str_pos_start);

						// Если тема закодирована и явно указана кодирова
						$subject_code = isset($aSubject[2][0]) ? $aSubject[2][0] : $this->_aMessages[$i]['code'];

						for ($n = 0; $n < count($aSubject[1]) - 1; $n++)
						{
							$this->_aMessages[$i]['subject'] .= trim($this->_headerDecode($aSubject[1][$n], $subject_code));

							$str_pos_start = mb_strpos($value->subject, $aSubject[1][$n]) + mb_strlen($aSubject[1][$n]);
							$this->_aMessages[$i]['subject'] .= trim(mb_substr($value->subject, $str_pos_start, mb_strpos($value->subject, $aSubject[1][$n + 1]) - $str_pos_start));
						}

						$this->_aMessages[$i]['subject'] .= $this->_headerDecode($aSubject[1][$n], $subject_code);
					}
					// Тема не закодирована, или состоит из одной части
					else
					{
						$this->_aMessages[$i]['subject'] = $this->_headerDecode($value->subject, $this->_aMessages[$i]['code']);
					}
				}
				elseif (isset($header_message->subject))
				{
					// Тема письма
					$this->_aMessages[$i]['subject'] = $this->_headerDecode($header_message->subject, $this->_aMessages[$i]['code']);
				}
				else
				{
					// Тема письма
					$this->_aMessages[$i]['subject'] = '';
				}

				// UID письма
				$this->_aMessages[$i]['uid'] = $value->uid;
			}

			$i++;
		}

		// Удалить письма после просмотра
		$this->delete && $this->_deleteMessages($this->_stream);

		imap_close($this->_stream);

		return $this;
	}

	/**
	 * Разбор структуры сообщения и сохранение результата в массив
	 *
	 * @param array $structure Массив со структурой сообщения
	 * @return array
	 */
	protected function _structure2array($structure)
	{
		$aStructureParts = array();
		$j = 0;

		if (isset($structure->parts))
		{
			// Разбираем структуру сообщения
			// [parts][0][]
			foreach ($structure->parts as $part_num => $part_param)
			{
				// [parts][0][type]
				foreach ($part_param as $attribute => $attributeValue)
				{
					if (is_array($attributeValue))
					{
						// [parts][0][parameters][]
						foreach ($attributeValue as $parameter_num => $parameter_values)
						{
							if (!is_array($parameter_values))
							{
								// [parts][0][parameters][0][attribute]
								foreach ($parameter_values as $parameter_values_part_num => $parameter_values_part_values)
								{
									if (is_object($parameter_values_part_values))
									{
										$aStructureParts[$j][mb_strtolower($attribute)][mb_strtolower(strval($parameter_values->attribute))] = strval($parameter_values->value);
									}
								}
							}
						}
					}
					elseif(is_object($attributeValue))
					{
						foreach ($attributeValue as $parameter_values_part_num => $parameter_values_part_values)
						{
							if (is_array($parameter_values_part_values))
							{
								$aStructureParts[$j][mb_strtolower($attribute)][mb_strtolower(strval($parameter_values_part_num))] = $parameter_values_part_values;
							}
							else
							{
								$aStructureParts[$j][mb_strtolower($attribute)][mb_strtolower(strval($attributeValue->attribute))] = strval($attributeValue->value);
							}
						}
					}
					else
					{
						$aStructureParts[$j][mb_strtolower($attribute)] = $attributeValue;
					}
				}

				if (isset($part_param->parts))
				{
					$aStructureParts[$j]['parts'] = $this->_structure2array($part_param);
				}

				$j++;
			}
		}

		return $aStructureParts;
	}

	/**
	 * Разбор тела сообщения
	 *
	 * @param string $imap_body Тело сообщения
	 * @param string $bound Разделитель между частями сообщения
	 * @param array $aStructureParts Структура сообщения
	 * @return array Массив с заголовками и телом частей сообщения
	 */
	protected function _parseHeaderAndBody($imap_body, $bound, $aStructureParts)
	{
		$imap_body = strval($imap_body);

		// Разбиваем тело сообщения на части по разделителю --$bound (при разделении сообщения, в его теле к заявленному в структуре $bound приписывается два тире)
		$imap_body = explode("--{$bound}", $imap_body);
		$j = 0;
		$part = array();

		foreach ($imap_body as $key_part => $body_part)
		{
			$body_part = trim(str_replace("\r", "", strval($body_part)));

			if ($body_part != '' && $body_part != "--")
			{
				if (isset($aStructureParts[0]['parameters']['boundary']) && (mb_strpos($body_part, $aStructureParts[0]['parameters']['boundary']) !== FALSE))
				{
					$part[$j] = $this->_parseHeaderAndBody($body_part, $aStructureParts[0]['parameters']['boundary'], $arr = array());
					$j++;
				}
				else
				{
					// Пустая строка - разделитель между заголовком и телом части сообщения
					$start_pos = mb_strpos($body_part, "\n\n");
					if ($start_pos !== FALSE)
					{
						$body_part_array = array();
						$body_part_array[0] = mb_substr($body_part, 0, $start_pos);
						$body_part_array[1] = mb_substr($body_part, $start_pos);

						// Если тело части сообщения не пустое
						if (trim($body_part_array[1]) != '')
						{
							$part[$j]['header'] = trim($body_part_array[0]);
							$part[$j]['body'] = trim($body_part_array[1]);
							$j++;
						}
					}
				}
			}
		}

		return $part;
	}

	/**
	 * Сохранение структуры сообщения
	 *
	 * @param array $aStructureParts Массив со структурой
	 * @return str
	 */
	protected function _saveStructure($aStructureParts)
	{
		if (is_array($aStructureParts) && count($aStructureParts))
		{
			foreach ($aStructureParts as $key => $val)
			{
				if (is_array($val) || is_object($val))
				{
					$this->_saveStructure($val);
				}
				else
				{
					$this->_headers .= "\n{$key}: {$val}";
				}
			}
		}
		return $this->_headers;
	}

	/**
	 * Разбор сообщения по частям
	 *
	 * @param int $i Порядковый номер письма в ящике
	 * @param string $imap_body Тело сообщения
	 * @param string $bound Разделитель между различными частями тела сообщения
	 */
	protected function _parseMessage($i, $imap_body, $bound, $is_first = 0)
	{
		// Метаданные сообщения
		$structure = imap_fetchstructure($this->_stream, $i + 1);

		// Если сообщение состоит из нескольких частей, формируем массив, каждый элемент которого будет содержанием соответствующей части сообщения
		$aStructureParts = $this->_structure2array($structure);

		// print_r($aStructureParts);
		// echo $imap_body;

		// Разбиваем тело сообщения на части по разделителю --$bound
		$part = $this->_parseHeaderAndBody($imap_body, $bound, $aStructureParts);

		// print_r($part);
		if (isset($aStructureParts[0]['parameters']['charset']))
		{
			$this->_aMessages[$i]['code'] = $aStructureParts[0]['parameters']['charset'];
		}
		elseif (isset($aStructureParts[0]['parts'][0]['parameters']['charset']))
		{
			$this->_aMessages[$i]['code'] = $aStructureParts[0]['parts'][0]['parameters']['charset'];
		}
		else
		{
			$this->_aMessages[$i]['code'] = '';
		}

		$this->_headers = '';
		$this->_aMessages[$i]['structure'] = $this->_saveStructure($aStructureParts);
		$this->_aMessages[$i]['structure_array'] = $aStructureParts;

		// Индекс элемента массива вложений
		$n = 0;

		// Разбираем каждую часть сообщения
		foreach ($part as $key => $value)
		{
			// Текст сообщения
			if ($key === 0)
			{
				if (isset($value[0]['body']))
				{
					$body = $value[0]['body'];
					$header = $value[0]['header'];

					// Если в Первой части сообщения указано на формат сообщения multi-part message in mime format, то текст сообщения берем из следующей части тела
					if (strstr(mb_strtolower($body), 'this is a multi-part message in mime format.') && isset($value[1]['body']))
					{
						$body = $value[1]['body'];
						$header = $value[1]['header'];
					}
				}
				elseif (isset($value['body']))
				{
					$body = $value['body'];
					$header = $value['header'];
				}
				elseif (isset($value[0][0]['body']))
				{
					$body = $value[0][0]['body'];
					$header = $value[0][0]['header'];
				}
				else
				{
					$body = '';
					$header = '';
				}

				if (isset($aStructureParts[$key]['parts'][0]))
				{
					// Подтип сообщения
					if (isset($aStructureParts[$key]['parts'][0]['subtype']))
					{
						$this->_aMessages[$i]['subtype'] = $aStructureParts[$key]['parts'][0]['subtype'];
					}
					else
					{
						$this->_aMessages[$i]['subtype'] = 'text';
					}

					if (isset($aStructureParts[$key]['parts'][0]['encoding']))
					{
						$text = $this->_bodyDecode(trim($body), $aStructureParts[$key]['parts'][0]['encoding']);
					}
					else
					{
						$text = trim($value['body']);
					}
				}
				else
				{
					// Подтип сообщения
					if (isset($aStructureParts[$key]['subtype']))
					{
						$this->_aMessages[$i]['subtype'] = $aStructureParts[$key]['subtype'];
					}
					else
					{
						$this->_aMessages[$i]['subtype'] = 'text';
					}

					if (isset($aStructureParts[$key]['encoding']))
					{
						$text = $this->_bodyDecode(trim($body), $aStructureParts[$key]['encoding']);
					}
					else
					{
						$text = trim($value['body']);
					}
				}

				if (isset($this->_aMessages[$i]['code']) && $this->_aMessages[$i]['code'] != '')
				{
					$text = $this->_iconv($text, $this->_aMessages[$i]['code']);
				}

				$this->_aMessages[$i]['body'] = $text;
			}
			// Вложенный файл
			elseif (isset($aStructureParts[$key]['dparameters']['filename*0*'])
			|| isset($aStructureParts[$key]['dparameters']['filename'])
			|| isset($aStructureParts[$key]['dparameters']['filename*'])
			|| (isset($aStructureParts[$key]['subtype'])
				&& in_array(mb_strtolower($aStructureParts[$key]['subtype']), array('gif', 'jpg', 'jpeg', 'vnd.ms-excel', 'msword')))
			|| (isset($aStructureParts[$key]['disposition'])
				&& mb_strtolower($aStructureParts[$key]['disposition']) == 'attachment'))
			{
				// Тип вложения
				if (isset($aStructureParts[$key]['type']))
				{
					$this->_aMessages[$i]['attachments'][$n]['type'] = trim($aStructureParts[$key]['type']);
				}

				// Имя вложения
				if (isset($aStructureParts[$key]['dparameters'])
				|| isset($aStructureParts[$key]['ifdparameters'])
				|| isset($aStructureParts[$key]['name']))
				{
					$file_name = '';

					if (isset($aStructureParts[$key]['dparameters']))
					{
						foreach ($aStructureParts[$key]['dparameters'] as $dparameters_key => $dparameters)
						{
							if (mb_strpos(mb_strtolower($dparameters_key), 'filename') !== FALSE)
							{
								$file_name .= $dparameters;
							}
						}
					}
					// GIF или JPG inline
					elseif (isset($aStructureParts[$key]['parameters']['name']))
					{
						$file_name .= $aStructureParts[$key]['parameters']['name'];
					}
					elseif (isset($aStructureParts[$key]['name']))
					{
						$file_name .= $aStructureParts[$key]['name'];
					}
					elseif (isset($aStructureParts[$key]['ifparameters']['name']))
					{
						$file_name .= $aStructureParts[$key]['ifparameters']['name'];
					}
					elseif (isset($aStructureParts[$key]['ifdparameters']['filename*0*'])
					|| isset($aStructureParts[$key]['dparameters']['filename'])
					|| isset($aStructureParts[$key]['dparameters']['filename*']))
					{
						foreach ($aStructureParts[$key]['ifdparameters'] as $dparameters_key => $dparameters)
						{
							if (mb_strpos(mb_strtolower($dparameters_key), 'filename') !== FALSE)
							{
								$file_name .= $dparameters;
							}
						}
					}
					//echo "<br>==$file_name";

					// Т.к. у некоторых между частями пробел не стоит
					$file_name = str_replace('=?==?', '=?= =?', $file_name);

					$file_name_explode = explode(' ', $file_name);

					// Далее к имени файла будет дописываться информация
					$this->_aMessages[$i]['attachments'][$n]['name'] = '';

					// Цикл по всем частям названия файла, которые были разделены пробелом
					foreach ($file_name_explode as $file_name_n)
					{
						// Ищем кодировку в имени файла
						preg_match_all("'([^\'\"]*?)\'\'(\"([^\"]*?)\"|.*?)(\Z|;)'siu", $file_name_n, $code);
						preg_match_all("'=\?([^=]*?)\?(Q?B?)\??(=?)([^\?]*?)\?=(.*?)'siu", $file_name_n, $code_1);

						if (isset($code[1][0]) && $code[1][0] != '')
						{
							$file_name_code = $code[1][0];
							$file_name = urldecode($code[2][0]);
						}
						elseif (isset($code_1[1][0]) && $code_1[1][0] != '')
						{
							$file_name_code = $code_1[1][0];
							$file_name = $code_1[3][0] . $code_1[4][0] . $code_1[5][0];
						}

						// Тип кодирования по умолчанию 3
						$encoding = 3;

						// Если ыыл передан тип кодирования (B, Q)
						if (isset($code_1[2][0]) && $code_1[2][0] != '')
						{
							switch ($code_1[2][0]) {
								case 'Q':
									$encoding = 4;
								break;
								case 'B':
									$encoding = 3;
								break;
								default:
									isset($aStructureParts[$key]['encoding']) && $encoding = $aStructureParts[$key]['encoding'];
								break;
							}
						}

						// Портит =?windows-1251?B?7+vg8uXm7e7lIO/u8PP35e3o5SAucGRm?=
						// на     =?windows-1251?B?7 vg8uXm7e7lIO/u8PP35e3o5SAucGRm?=
						// перенесено выше для случая "UTF-8''%D0%B1%D0%BB%D0%BE%D0%B3%2E%64%6F%63"
						//$file_name = urldecode($file_name);
						$decoded_str = $this->_bodyDecode($file_name, 4);

						// Если декодированная строка аналогична переданной
						if ($decoded_str == $file_name || $decoded_str . "=" == $file_name)
						{
							if (preg_match("'[!;:%\*\(\),\.]'u", $decoded_str))
							{
								$decoded_str = $file_name;
							}
							else
							{
								$decoded_str = $this->_bodyDecode($file_name, $encoding);
							}
						}

						if (isset($file_name_code) && trim($file_name_code) != '')
						{
							$decoded_str = $this->_iconv($decoded_str, $file_name_code);
						}

						// Дописываем новую часть имени файла
						$this->_aMessages[$i]['attachments'][$n]['name'] .= $decoded_str;
					}
				}

				// Тело вложения
				if (isset($aStructureParts[$key]['encoding']))
				{
					$file_body = $this->_bodyDecode(trim($value['body']), $aStructureParts[$key]['encoding']);
				}
				else
				{
					$file_body = trim($value['body']);
				}

				/*if (isset($aStructureParts[0]['parameters']['charset']))
				 {
				 $file_body = $this->_iconv($file_body, $aStructureParts[0]['parameters']['charset']);
				 }*/

				$this->_aMessages[$i]['attachments'][$n]['body'] = $file_body;

				$n++;
			}
			else
			{
				// НЕ ФАЙЛ
			}
		}
	}

	/**
	 * Удаление всех писем из почтового ящика
	 * @param resource $stream Идентификатор открытого соединения с почтовым сервером
	 */
	protected function _deleteMessages($stream)
	{
		// Количество писем в почтовом ящике
		$iCount = imap_num_msg($stream);

		// Помечаем на удаление все письма в ящике
		for ($i = 0; $i < $iCount; $i++)
		{
			imap_delete($stream, $i + 1);
		}

		// Удаляем письма, помеченные на удаление
		imap_expunge($stream);

		return $this;
	}

	/**
	 * Перекодировщик тела письма
	 *
	 * @param string $str Закодированная строка
	 * @param string $encoding_type Кодировка
	 * @return string Раскодированная строка
	 */
	protected function _bodyDecode($str, $encoding_type = '')
	{
		if (trim($str) == '')
		{
			return $str;
		}

		// Письмо закодировано методом base64
		if (mb_strpos(mb_strtolower($encoding_type), "base") !== FALSE
		|| $encoding_type == 3
		|| $encoding_type == 2)
		{
			if ($encoding_type == 2)
			{
				$str = imap_binary($str);
			}
			if ($this->_isBase64($str))
			{
				$str = base64_decode($str);
			}
		}
		// Письмо закодировано методом quoted-printable
		elseif (strpos(strtolower($encoding_type), "quoted") !== FALSE
		/*|| mb_strpos(mb_strtolower($encoding_type), "bit") !== FALSE*/
		|| $encoding_type == 4
		/*|| $encoding_type == 1*/)
		{
			$str = quoted_printable_decode($str);
		}

		return $str;
	}

	/**
	 * Проверка строки на то, является ли она закодированной base64
	 *
	 * @param $string строка
	 * @return boolean
	 */
	protected function _isBase64($string)
	{
		if (empty($string))
		{
			return FALSE;
		}

		$length = mb_strlen($string);

		// Длина строки должна быть кратна 4
		// Исключено, т.к. могут быть переводы строк, которые меняют шаг
		/*if (($length % 4) > 0)
		{
			return FALSE;
		}*/

		// Проверка на равно, который разрешен в начале или в конце строки
		if (mb_strpos(mb_substr($string, 2, ($length - 4)),  '=') > 0)
		{
			return FALSE;
		}

		return true;
	}

	/**
	 * Перекодировщик из кодировки письма в кодировку системы
	 *
	 * @param string $str Закодированная строка
	 * @param string $code Кодировка
	 * @return str
	 */
	protected function _headerDecode($str, $code = '')
	{
		// Передана пустая строка
		if ($str == '')
		{
			return $str;
		}

		// Декодирование MIME-заголовков сообщения
		$str_decoded = @imap_mime_header_decode($str);

		// Составляем декодированную строку
		foreach ($str_decoded as $obj)
		{
			$arr = $obj->text;
		}

		return $this->_iconv($arr, $code);
	}

	/**
	 * Перекодирование текста из кодировки письма в кодировку системы
	 *
	 * @param string $text Текст
	 * @param string $sourceCode Исходная кодировка
	 * @return string
	 */
	protected function _iconv($text, $sourceCode)
	{
		$text = strval($text);
		$sourceCode = trim(strtolower($sourceCode));

		if (strstr($sourceCode, 'utf'))
		{
			$sourceCode = 'utf-8';
		}
		elseif (strstr($sourceCode, 'koi8-r'))
		{
			$sourceCode = 'koi8-r';
		}
		elseif (strstr($sourceCode, 'koi8-u'))
		{
			$sourceCode = 'koi8-u';
		}
		elseif (strstr($sourceCode, 'windows-1251'))
		{
			$sourceCode = 'windows-1251';
		}

		if ($sourceCode != '' && $sourceCode != 'utf-8' && $sourceCode != 'x-unknown')
		{
			$text = @iconv($sourceCode, 'UTF-8//IGNORE', $text);
		}

		return $text;
	}

	/**
	 * Сохранение заголовков письма
	 *
	 * @param array $headers Массив заголовков
	 * @param string $code Кодировка
	 * @return string Строка заголовков
	 */
	protected function _saveHeaders($headers, $code)
	{
		if (is_array($headers))
		{
			foreach ($headers as $header => $value)
			{
				// Если есть подзаголовки
				if (is_array($value))
				{
					$array = array();
					foreach ($value as $v)
					{
						$array[] = $v;
					}

					$this->_saveHeaders($array, $code);
				}
				else
				{
					$this->_headers .= $header . ': ' . (
						strstr($value, '=?')
							? $this->_headerDecode($value, $code)
							: $this->_iconv($value, $code)
					) . "\n";
				}
			}
		}

		return $this->_headers;
	}
}