<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Online shop.
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Shop_Item_Controller extends Core_Servant_Properties
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array(
		'count',
		'siteuser'
	);

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		parent::__construct();

		// Количество товара по умолчанию равно 1
		$this->count = 1;
	}

	/**
	 * Определение цены товара для заданного пользователя $this->siteuser
	 *
	 * @param Shop_Item_Model $oShop_Item товар
	 * @return array возвращает массив значений цен для данного пользователя
	 * - $price['tax'] сумма налога
	 * - $price['rate'] размер налога
	 * - $price['price'] цена с учетом валюты без налога
	 * - $price['price_tax'] цена с учетом налога
	 * - $price['price_discount'] цена с учетом налога и со скидкой
	 */
	public function getPrices(Shop_Item_Model $oShop_Item)
	{
		if (is_null($oShop_Item->id))
		{
			throw new Core_Exception('Shop_Item_Controller::getPrices Shop_Item id is NULL.');
		}

		$oShop = $oShop_Item->Shop;

		$price = array();
		$price['tax'] = 0;
		$price['rate'] = 0;
		$price['price'] = $oShop_Item->price;

		// Пользователь задан - цена определяется из таблицы товаров
		if ($this->siteuser && Core::moduleIsActive('siteuser'))
		{
			$aPrices = array();

			$aSiteuser_Groups = $this->siteuser->Siteuser_Groups->findAll();
			foreach ($aSiteuser_Groups as $oSiteuser_Group)
			{
				$oShop_Price = Core_Entity::factory('Shop_Price')->getBySiteuserGroupAndShop($oSiteuser_Group->id, $oShop->id);

				// Если есть цена для группы
				if ($oShop_Price)
				{
					// Смотрим, определена ли такая цена для данного товара
					$oShop_Item_Price =$oShop_Item->Shop_Item_Prices->getByPriceId($oShop_Price->id);

					if ($oShop_Item_Price)
					{
						$aPrices[] = $oShop_Item_Price->value;
					}
				}
			}

			count($aPrices) > 0 && $price['price'] = min($aPrices);
		}

		// Цены в зависимости от количества самого товара в корзине (а не все корзины)
		$aShop_Specialprices = $oShop_Item->Shop_Specialprices->findAll();
		foreach ($aShop_Specialprices as $oShop_Specialprice)
		{
			if ($this->count >= $oShop_Specialprice->min_quantity && $this->count <= $oShop_Specialprice->max_quantity)
			{
				$price['price'] = $oShop_Specialprice->percent != 0
					? $price['price'] * $oShop_Specialprice->percent / 100
					: $oShop_Specialprice->price;
				break;
			}
		}

		// Определяем коэффициент пересчета
		$fCurrencyCoefficient = $oShop_Item->Shop_Currency->id > 0 && $oShop->Shop_Currency->id > 0
			? Shop_Controller::instance()->getCurrencyCoefficientInShopCurrency(
				$oShop_Item->Shop_Currency, $oShop->Shop_Currency
			)
			: 0;

		// Умножаем цену товара на курс валюты в базовой валюте
		$price['price'] = $price['price'] * $fCurrencyCoefficient;

		// Выбираем информацию о налогах
		if ($oShop_Item->shop_tax_id)
		{
			// Извлекаем информацию о налоге
			$oShop_Tax = $oShop_Item->Shop_Tax;

			if ($oShop_Tax->id)
			{
				$price['rate'] = $oShop_Tax->rate;

				// Если он не входит в цену
				if ($oShop_Tax->tax_is_included == 0)
				{
					// То считаем цену с налогом
					$price['tax'] = $oShop_Tax->rate / 100 * $price['price'];
					$price['price_tax'] = $price['price'] + $price['tax'];
				}
				else
				{
					$price['tax'] = $price['price'] / (100 + $oShop_Tax->rate) * $oShop_Tax->rate;
					$price['price_tax'] = $price['price'];
				}
			}
		}
		else
		{
			$price['price_tax'] = $price['price'];
		}

		$price['discounts'] = array();

		// Определены ли скидки на товар
		$aShop_Item_Discounts = $oShop_Item->Shop_Item_Discounts->findAll();
		if (count($aShop_Item_Discounts))
		{
			// определяем количество скидок на товар
			$percent = 0;

			// Цикл по идентификаторам скидок для товара
			foreach ($aShop_Item_Discounts as $oShop_Item_Discount)
			{
				if ($oShop_Item_Discount->Shop_Discount->isActive())
				{
					$price['discounts'][] = $oShop_Item_Discount->Shop_Discount;
					$percent += $oShop_Item_Discount->Shop_Discount->percent;
				}
			}

			// определяем суммарную величину скидки в валюте
			$price['discount'] = $price['price_tax'] * $percent / 100;

			// вычисляем цену со скидкой как ее разность с величиной скидки
			$price['price_discount'] = $price['price_tax'] - $price['discount'];
		}
		else
		{
			// если скидок нет, то price_discount положим равным price
			$price['price_discount'] = $price['price_tax'];
			$price['discount'] = 0;
		}

		$oShop_Controller = Shop_Controller::instance();

		// Округляем значения, переводим с научной нотации 1Е+10 в десятичную
		$price['tax'] = $oShop_Controller->round($price['tax']);
		$price['price'] = $oShop_Controller->round($price['price']);
		$price['price_discount'] = $oShop_Controller->round($price['price_discount']);
		$price['price_tax'] = $oShop_Controller->round($price['price_tax']);

		return $price;
	}
}